"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ChatComponent = void 0;
var _react = _interopRequireWildcard(require("react"));
var _Widget = require("./components/Widget");
var _Chat = require("./components/Chat");
var _entities = require("./entities");
var _types = require("./types");
var _global = require("./global");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const ChatComponent = _ref => {
  var _configurationsState$;
  let {
    configurations,
    ws,
    sessionGateway,
    messagesGateway,
    configurationsGateway,
    appearance
  } = _ref;
  const [sessionState, setSessionState] = (0, _react.useState)(null);
  const [messageState, setMessageState] = (0, _react.useState)(new _entities.MessageList([]));
  const wsRef = (0, _react.useRef)(null);
  const onIncomingMessage = (data, from) => {
    const newMessageList = new _entities.MessageList(messageState === null || messageState === void 0 ? void 0 : messageState.messages);
    newMessageList.observers = (messageState === null || messageState === void 0 ? void 0 : messageState.observers) || [];
    newMessageList.addMessage({
      ...data,
      id: data.id || `${Date.now() * Math.random()}`,
      from
    });
    setMessageState(newMessageList);
  };
  sessionGateway.onEndConversation(async () => {
    await sessionGateway.clearSession();
  });
  messagesGateway.onMessage(msg => onIncomingMessage(msg, _types.From.BOT));
  (0, _react.useEffect)(() => {
    if (!wsRef.current) {
      wsRef.current = ws;
      wsRef.current.connect();
      wsRef.current.onOpen(async () => {
        await sessionGateway.linkSession();
      });
    }
  }, [ws, sessionGateway]);
  (0, _react.useEffect)(() => {
    async function loadData() {
      var _wsRef$current;
      const [storedSession, storedMessages] = await Promise.all([sessionGateway.getCurrent(), messagesGateway.getMessages()]);
      storedSession.register(new _entities.Observer('sendAction', async data => {
        await sessionGateway.sendAction(data);
      }));
      storedSession.register(new _entities.Observer('clearSession', async () => {
        await sessionGateway.clearSession();
        const newSession = await sessionGateway.subscribe('');
        newSession.observers = storedSession.observers;
        setSessionState(newSession);
      }));
      wsRef === null || wsRef === void 0 ? void 0 : (_wsRef$current = wsRef.current) === null || _wsRef$current === void 0 ? void 0 : _wsRef$current.onLink(async () => {
        await sessionGateway.subscribe(storedSession.current);
      });
      setSessionState(storedSession);
      storedMessages.register(new _entities.Observer('addMessage', async data => {
        if (data.type !== _types.MessageTypes.TYPING) {
          if (data.from === _types.From.USER) {
            await messagesGateway.sendMessage(data);
          } else {
            await messagesGateway.sendStatus(data.id, 'delivered');
            await messagesGateway.storeMessage(data);
          }
        }
      }));
      setMessageState(storedMessages);
    }
    loadData();
  }, [sessionGateway, messagesGateway]);
  const restartConversation = () => {
    const messageList = new _entities.MessageList([]);
    messageList.observers = (messageState === null || messageState === void 0 ? void 0 : messageState.observers) || [];
    setMessageState(messageList);
    messageState === null || messageState === void 0 ? void 0 : messageState.clearMessages();
    sessionState === null || sessionState === void 0 ? void 0 : sessionState.clearSession();
  };
  const [configurationsState, setConfigurationsState] = (0, _react.useState)(configurations);
  (0, _react.useEffect)(() => {
    const loadConfigurations = async () => {
      const configurationsResponse = await configurationsGateway.getStyles(_global.Global.botId, appearance);
      setConfigurationsState(new _entities.ChatConfigurations(configurationsResponse.title, configurationsResponse.poweredBy, configurationsResponse.poweredByUrl, configurationsResponse.settings, configurationsState.isOpen));
    };
    loadConfigurations();
  }, [configurationsGateway, _global.Global.botId]);
  const toggleIsChatOpen = () => {
    const updatedConfigurations = new _entities.ChatConfigurations(configurationsState.title, configurationsState.poweredBy, configurationsState.poweredByUrl, configurationsState.settings, configurationsState.isOpen);
    updatedConfigurations.toggleIsOpen();
    setConfigurationsState(updatedConfigurations);
  };
  const closeChat = () => {
    const updatedConfigurations = new _entities.ChatConfigurations(configurationsState.title, configurationsState.poweredBy, configurationsState.poweredByUrl, configurationsState.settings, configurationsState.isOpen);
    updatedConfigurations.close();
    setConfigurationsState(updatedConfigurations);
  };
  return /*#__PURE__*/_react.default.createElement(_react.default.Fragment, null, /*#__PURE__*/_react.default.createElement(_Chat.Chat, {
    messages: (messageState === null || messageState === void 0 ? void 0 : messageState.messages) || [],
    configurations: configurationsState,
    close: closeChat,
    restartConversation: restartConversation,
    sendMessage: msg => onIncomingMessage(msg, _types.From.USER)
  }), /*#__PURE__*/_react.default.createElement(_Widget.Widget, {
    onPress: toggleIsChatOpen,
    imageUrl: configurationsState === null || configurationsState === void 0 ? void 0 : (_configurationsState$ = configurationsState.settings) === null || _configurationsState$ === void 0 ? void 0 : _configurationsState$.botFab
  }));
};
exports.ChatComponent = ChatComponent;
//# sourceMappingURL=ChatComponent.js.map